
# read alignment files and construct bin-level files

constructBins <- function( infileLoc=NULL, infileName=NULL, fileFormat=NULL, outfileLoc=infileLoc, fragLen=200, binSize=fragLen, capping=3, perl = "perl" )
{   
    # preprocessing perl script embedded in "mosaics/inst/Perl/" (for SET)
    
    script <- "process_readfiles.pl"
    allFormat <- c( "eland_result", "eland_extended", "eland_export", "bowtie", "sam" )
    allFormatName <- c( "Eland result", "Eland extended", "Eland export", "Bowtie default", "SAM" )
    
    # Check whether perl exists
    
    CMD <- paste( perl, "-v" )
    res <- system( CMD, intern = TRUE, ignore.stderr = TRUE )
  
    if ( length(res) == 0 ) {
    	# cannot proceed if perl does not exist
    	
    	stop( "Perl is not found on your system! Either check $PATH if installed or please install Perl." )
    } else {
	# process read files into bin-level files if perl exists
	
	# check whether minimal options are missing
	
	if ( length(infileLoc) != 1 || is.null(infileLoc) )
	{
		stop( "Please specify the location of the aligned read file!" )
	}	
	
	if ( length(infileName) != 1 || is.null(infileName) )
	{
		stop( "Please specify the name of the aligned read file!" )
	}		
	
	if ( length(fileFormat) != 1 || is.null(fileFormat) )
	{
		stop( "Please specify aligned read file format! Read '?constructBins' for supported file formats" )
	}	
	
	# check file format specification
	
	if ( length(which(!is.na(match( fileFormat, allFormat )))) == 0 )
	{
		stop( "Unsupported aligned read file format! Read '?constructBins' for supported file formats" )
	}
		
	
	# print out processing settings:
	# by default, set fragment length = 200, bin size = fragment length, capping = 3.
	
	fileFormatName <- allFormatName[ match( fileFormat, allFormat ) ]
	
	currentLoc <- getwd()
	setwd(infileLoc)
	infileSizeB <- file.info( infileName )$size
	setwd(currentLoc)
	if ( infileSizeB/1024^3 > 1 ) {
		infileSize <- paste( round(infileSizeB/1024^3),"G",sep="" )
	} else if ( infileSizeB/1024^2 > 1 ) {
		infileSize <- paste( round(infileSizeB/1024^2),"M",sep="" )
	} else {
		infileSize <- paste( round(infileSizeB/1024),"K",sep="" )
	}
	
	cat( "------------------------------------------------------------\n" )
	cat( "Info: setting summary\n" )
	cat( "------------------------------------------------------------\n" )
	cat( "Directory of aligned read file:", infileLoc, "\n" )
	cat( "Name of aligned read file: ",infileName," (",infileSize,")\n", sep="" )
	cat( "Aligned read file format:", fileFormatName, "\n" )
	cat( "Directory of processed bin-level files:", outfileLoc, "\n" )
	cat( "Fragment length:", fragLen, "\n" )
	cat( "Bin size:", binSize, "\n" )
	cat( "Maximum number of reads allowed in each nucleotide:", capping, "\n" )
	cat( "------------------------------------------------------------\n" )
	
	
	# get path to the perl code (unified script for all file formats)
	
	require(mosaics)
    	#Perl.Path <- file.path( .path.package("mosaics"), "Perl" )
    	#Fn.Path <- file.path( Perl.Path, script )
    	Fn.Path <- system.file( file.path("Perl",script), package="mosaics")
	
	
	# process read file to bin-level files using perl codes
	
	message( "Info: reading the aligned read file and processing it into bin-level files..." )
	
	CMD <- paste( perl, 
	#	" -I", shQuote(Perl.Path),
	#	" ", shQuote(Fn.Path),
		" ", Fn.Path,
		" ", fileFormat, 
		" ", fragLen, 
		" ", binSize, 
		" ", capping, 
		" ", infileLoc, 
		" ", outfileLoc, 
		" ", infileName, sep="" )
    	
    	start_time <- proc.time()
    	res <- system( CMD, intern = TRUE )
    	elapsed_time <- as.vector( proc.time() - start_time )[3]
    	
    	if ( elapsed_time < 60 ) {
    		message( paste("Info: construction of bin-level files takes",
    			elapsed_time,"seconds.") )
    	} else {
    		message( paste("Info: construction of bin-level files takes",
    			round(elapsed_time/60),"minutes.") )
    	}
	
	#if ( res != 0 ) {
	#	# error occurs from the perl script
	#	
	#	stop( paste("The perl script '",script,"' failed to run successfully!",sep="") )
	#}
	
	message( "Info: done!" )
	
	
	# print out processing results
	
	currentLoc <- getwd()
	setwd(outfileLoc)
	outfileName <- system( paste("ls *_",infileName,"_fragL",fragLen,"_bin",binSize,".txt",sep=""),
		intern=TRUE )
	setwd(currentLoc)
	
	cat( "------------------------------------------------------------\n" )
	cat( "Info: processing summary\n" )
	cat( "------------------------------------------------------------\n" )
	cat( "Directory of processed bin-level files:", outfileLoc, "\n" )
	cat( "List of processed bin-level files:\n" )
	for ( i in 1:length(outfileName) ) {		
		currentLoc <- getwd()
		setwd(outfileLoc)
		outfileSizeB <- file.info( outfileName[i] )$size
		setwd(currentLoc)
		if ( outfileSizeB/1024^3 > 1 ) {
			outfileSize <- paste( round(outfileSizeB/1024^3),"G",sep="" )
		} else if ( outfileSizeB/1024^2 > 1 ) {
			outfileSize <- paste( round(outfileSizeB/1024^2),"M",sep="" )
		} else {
			outfileSize <- paste( round(outfileSizeB/1024),"K",sep="" )
		}
		
		cat( "- ",outfileName[i]," (",outfileSize,")\n", sep="" )
	}
	cat( "------------------------------------------------------------\n" )
    }
}
